function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

//Dependencies
import * as validate from "./validators.js";
/*
 * Panda JS
*/

class Builder {
  //Constructor
  constructor(slides, userOptions) {
    //Build our default options
    this.slides = slides; //Build our default options

    this.options = _objectSpread(_objectSpread({}, this.buildDefaultOptions(userOptions)), userOptions);
    this.resolver = '';
    this.results = {
      complete: false,
      inputs: []
    };
  }
  /**
   * Create a new panda pop
   * 
   * @return {Promise}
  */


  create() {
    var _this = this;

    return _asyncToGenerator(function* () {
      //Promise
      return new Promise( /*#__PURE__*/function () {
        var _ref = _asyncToGenerator(function* (resolve, reject) {
          _this.resolver = resolve; //Build it

          _this.panda = yield _this.build(); //Add our slides

          for (var [key, slide] of Object.entries(_this.slides)) {
            yield _this.addSlide(key, slide);
          } //Configure the options


          yield _this.configureOptions(); //Insert on the page

          document.querySelector('body').append(_this.panda); //Open

          yield _this.open(); //Run our bindings

          yield _this.bindings();
        });

        return function (_x, _x2) {
          return _ref.apply(this, arguments);
        };
      }());
    })();
  }
  /**
   * Build the default options
   * 
   * @param {Object} userOptions The user options for any overwrites
   * @return {Object} An object of options
  */


  buildDefaultOptions(userOptions) {
    var options = {
      closeBtn: true,
      position: 'center'
    }; //Return

    return options;
  }
  /**
   * Build the panda
   * 
   * @return {Promise} HTMLElement The new panda
  */


  build() {
    var _this2 = this;

    return _asyncToGenerator(function* () {
      //Build the new Select HTML element and add it directly after the target
      var wrapper = document.createElement('div'),
          container = document.createElement('div'); //Set attributes

      container.classList.add('container');
      wrapper.classList.add('pandajs');
      wrapper.setAttribute('id', "pandajs-".concat(Math.random().toString(36).substr(2, 4))); //Position

      wrapper.classList.add(_this2.options.position); //Close Btn?

      if (_this2.options.closeBtn) {
        var btn = document.createElement('span');
        btn.classList.add('close');
        btn.innerHTML = yield _this2.getIconContent('close');
        container.append(btn);
      } //Populate


      wrapper.appendChild(container); //Return

      return wrapper;
    })();
  }
  /**
   * Configure options for the panda
   * 
   * @return {Promise}
  */


  configureOptions() {
    return _asyncToGenerator(function* () {
      //Return
      return;
    })();
  }
  /**
   * Open the panda
   * 
   * @return {Promise}
  */


  open() {
    var _this3 = this;

    return _asyncToGenerator(function* () {
      //Open it (with a javascript animation to avoid css transform issues)
      var container = _this3.panda.querySelector('.container');

      yield container.animate([{
        transform: 'scale(0)'
      }, {
        transform: 'scale(1.2)'
      }, {
        transform: 'scale(1)'
      }], {
        duration: 150,
        iterations: 1
      }); //Return

      return;
    })();
  }
  /**
   * Add a slide
   * 
   * @param {Object} key - The key
   * @param {Object} slide - A slide object
   * @returns {Promise}
  */


  addSlide(key, slide) {
    var _this4 = this;

    return _asyncToGenerator(function* () {
      //Slide wrapper and content
      var wrapper = document.createElement('div'),
          header = document.createElement('div'),
          body = document.createElement('div'),
          footer = document.createElement('div'),
          [title, message, inputs, buttons] = yield Promise.all([_this4.buildTitle(slide), _this4.buildMessage(slide), _this4.buildFields(slide), _this4.buildButtons(slide)]); //Build the sections

      if (title) header.append(title);
      if (message) body.append(message);
      if (inputs) body.append(inputs);
      if (buttons) footer.append(buttons); //Attributes

      header.classList.add('header-wrapper');
      body.classList.add('body-wrapper');
      footer.classList.add('footer-wrapper'); //Append the sections to the slide

      if (header.hasChildNodes()) wrapper.append(header);
      if (body.hasChildNodes()) wrapper.append(body);
      if (footer.hasChildNodes()) wrapper.append(footer); //Attributes

      wrapper.classList.add('slide');
      wrapper.setAttribute('data-slide', parseInt(key) + 1); //Is this the primary one?

      if (key == 0) wrapper.classList.add('active'); //Add it to the instance

      _this4.panda.querySelector('.container').append(wrapper); //Build the content


      return;
    })();
  }
  /* ------------------------------------------------------------------------------------------------------------------------
      HTML Builders
  ------------------------------------------------------------------------------------------------------------------------ */

  /**
   * Build the title
   * 
   * @param {Object} slide - The slide we're building the title for
   * @returns {Promise}
  */


  buildTitle(slide) {
    return _asyncToGenerator(function* () {
      //Undefined?
      if (slide.title == undefined || slide.title == '') return false; //Create

      var title = document.createElement('h3'); //Set attributes and content

      title.innerHTML = slide.title; //Return

      return title;
    })();
  }
  /**
   * Build the message
   * 
   * @param {Object} slide - The slide we're building the title for
   * @returns {Promise}
  */


  buildMessage(slide) {
    return _asyncToGenerator(function* () {
      //Undefined?
      if (slide.message == undefined || slide.message == '') return false; //Create

      var message = document.createElement('p'); //Set attributes and content

      message.innerHTML = slide.message; //Return

      return message;
    })();
  }
  /**
   * Build the buttons
   * 
   * @param {Object} slide - The slide we're building the title for
   * @returns {Promise}
  */


  buildButtons(slide) {
    return _asyncToGenerator(function* () {
      //Undefined?
      if (slide.buttons == undefined || slide.buttons == '' || slide.buttons.length == 0) return false; //Create

      var wrapper = document.createElement('div'); //Set attributes and content

      wrapper.classList.add('controls-wrapper'); //Loop the buttons

      for (var btn of slide.buttons) {
        var button = document.createElement('button'); //Custom?

        if (btn.action == 'custom') {
          var customAction = btn.customAction;
          if (typeof customAction != 'function') continue;
        } //Populate


        button.innerHTML = btn.text;
        button.setAttribute('data-action', btn.action);
        if (!['', undefined].includes(btn.class)) button.classList.add(btn.class);
        if (!['', undefined].includes(btn.id)) button.setAttribute('id', btn.id); //Append it

        wrapper.appendChild(button);
      } //Return


      return wrapper;
    })();
  }
  /**
   * Build the inputs
   * 
   * @param {Object} slide - The slide we're building the inputs for
   * @returns {Promise}
  */


  buildFields(slide) {
    var _this5 = this;

    return _asyncToGenerator(function* () {
      //Undefined?
      if (slide.inputs == undefined) return false; //Create wrapper

      var wrapper = document.createElement('div'); //Set attributes and content

      wrapper.classList.add('inputs-wrapper'); //Loop the buttons

      for (var input of slide.inputs) {
        switch (input.type) {
          case 'text':
          case 'email':
          case 'number':
            wrapper.append(yield _this5.buildInput(input));
            break;

          case 'select':
            wrapper.append(yield _this5.buildSelect(input));
            break;
        }
      } //Return


      return wrapper;
    })();
  }
  /* ------------------------------------------------------------------------------------------------------------------------
      Input types
  ------------------------------------------------------------------------------------------------------------------------ */

  /**
   * General input (text, email, number, etc)
   * 
   * @param {Object} input - The input item
   * @returns {Promise}
  */


  buildInput(input) {
    return _asyncToGenerator(function* () {
      var field = document.createElement('input'); //Type

      field.setAttribute('type', input.type); //Set attributes

      if (!['', undefined].includes(input.class)) field.classList.add(input.class);
      if (!['', undefined].includes(input.id)) field.setAttribute('id', input.id);
      if (!['', undefined].includes(input.placeholder)) field.setAttribute('placeholder', input.placeholder);
      if (!['', undefined].includes(input.name)) field.setAttribute('name', input.name);
      if (!['', undefined].includes(input.value)) field.value = input.value;
      if (input.required === true) field.setAttribute('required', true); //Return

      return field;
    })();
  }
  /**
   * Select box
   * 
   * @param {Object} input - The input item
   * @returns {Promise}
  */


  buildSelect(input) {
    return _asyncToGenerator(function* () {
      var field = document.createElement('select'); //Set attributes

      if (!['', undefined].includes(input.class)) field.classList.add(input.class);
      if (!['', undefined].includes(input.id)) field.setAttribute('id', input.id);
      if (!['', undefined].includes(input.placeholder)) field.setAttribute('placeholder', input.placeholder);
      if (!['', undefined].includes(input.name)) field.setAttribute('name', input.name);
      if (input.required === true) field.setAttribute('required', true); //Loop the options and add

      for (var option of input.options) {
        var o = document.createElement('option');
        o.value = option.value;
        o.innerHTML = option.option;
        if (option.checked === true) o.setAttribute('selected', '');
        field.append(o);
      } //Return


      return field;
    })();
  }
  /* ------------------------------------------------------------------------------------------------------------------------
      Functions
  ------------------------------------------------------------------------------------------------------------------------ */

  /**
   * Bind next slide
   * 
   * @param {Object} slideObj - The slide
   * @param {Int} key - The key
   * @return {Promise}
  */


  nextSlideBtn(slideObj, key) {
    var _this6 = this;

    return _asyncToGenerator(function* () {
      //Try
      try {
        var _loop = function _loop(btn) {
          btn.addEventListener('click', /*#__PURE__*/_asyncToGenerator(function* () {
            //Get the current slide
            var slide = btn.closest(".slide[data-slide=\"".concat(key, "\"]")),
                nextNum = key + 1,
                nextSlide = _this6.panda.querySelector(".slide[data-slide=\"".concat(nextNum, "\"]")),
                inputs = slide.querySelectorAll('input[required], select[required], textarea[required]'); //Add a loader


            slide.classList.add('loading'); //Do we have required fields that are empty?

            var valid = true; //Loop the inputs

            for (var input of inputs) {
              //Reset errors
              input.removeAttribute('data-error'); //If required but empty, we dont need to type check

              if (input.value == '') {
                valid = false;
              } else {
                //Type check
                switch (input.getAttribute('type')) {
                  case 'email':
                    valid = yield validate.email(input.value);
                    break;

                  case 'tel':
                    valid = yield validate.telephone(input.value);
                    break;
                }
              } //If not valid, mark it


              if (!valid) input.setAttribute('data-error', true);
            } //Return if not valid


            if (!valid) {
              //Remove a loader
              slide.classList.remove('loading');
              return;
            } //Is there a prevalidate function?


            if (slideObj.preValidate) {
              var preValidateResult = yield slideObj.preValidate(slide);

              if (preValidateResult !== true) {
                //Remove a loader
                slide.classList.remove('loading');
                return;
              }
            } //Push our results


            for (var _input of inputs) {
              _this6.results.inputs[_input.getAttribute('name')] = _input.value;
            } //Do we have a next slide to work with or are we finished?


            if (nextSlide == undefined) return _this6.complete(); //Load the next slide

            slide.classList.remove('active');
            nextSlide.classList.add('active'); //Remove a loader

            slide.classList.remove('loading');
          }));
        };

        //Elems
        for (var btn of _this6.panda.querySelectorAll(".slide[data-slide=\"".concat(key, "\"] .footer-wrapper [data-action=\"next\"]"))) {
          _loop(btn);
        }
      } catch (e) {
        console.log('Error loading next slide', e);
      }
    })();
  }
  /**
   * Bind previous slide
   * 
   * @param {Object} slideObj - The slide
   * @param {Int} key - The key
   * @return {Promise}
  */


  previousSlideBtn(slideObj, key) {
    var _this7 = this;

    return _asyncToGenerator(function* () {
      //Try
      try {
        var _loop2 = function _loop2(btn) {
          btn.addEventListener('click', /*#__PURE__*/_asyncToGenerator(function* () {
            //Get the current slide
            var slide = btn.closest('.slide'),
                prevNum = key - 1,
                prevSlide = _this7.panda.querySelector(".slide[data-slide=\"".concat(prevNum, "\"]")); //Do we have a next slide to work with or are we finished?


            if (prevSlide == undefined) return _this7.cancel(); //Load the next slide

            slide.classList.remove('active');
            prevSlide.classList.add('active');
          }));
        };

        //Elems
        for (var btn of _this7.panda.querySelectorAll(".slide[data-slide=\"".concat(key, "\"] .footer-wrapper [data-action=\"previous\"]"))) {
          _loop2(btn);
        }
      } catch (e) {
        console.log('Error loading prev slide', e);
      }
    })();
  }
  /**
   * Bind custom slide btn
   * 
   * @param {Object} slideObj - The slide
   * @param {Int} key - The key
   * @return {Promise}
  */


  customSlideBtn(slideObj, key) {
    var _this8 = this;

    return _asyncToGenerator(function* () {
      //Try
      try {
        (function () {
          //Get panda instance
          var panda = _this8; //Elems

          for (var btn of _this8.panda.querySelectorAll(".slide[data-slide=\"".concat(key, "\"] .footer-wrapper [data-action=\"custom\"]"))) {
            btn.addEventListener('click', /*#__PURE__*/_asyncToGenerator(function* () {
              //Get the custom action
              var customAction = false; //Loop the slide obj buttons and find our custom one

              for (var button of slideObj.buttons) {
                if (button.action == 'custom' && button.customAction != undefined) {
                  customAction = button.customAction;
                }
              }

              customAction(slideObj, panda);
            }));
          }
        })();
      } catch (e) {
        console.log('Error loading custom slide btn', e);
      }
    })();
  }
  /**
   * Bind cancel btn
   * 
   * @return {Promise}
  */


  cancelBtn() {
    var _this9 = this;

    return _asyncToGenerator(function* () {
      //Try
      try {
        //Elems
        var btn = _this9.panda.querySelector('.container > span.close');

        btn.addEventListener('click', () => _this9.cancel());
      } catch (e) {
        console.log('Error cancel btn', e);
      }
    })();
  }
  /**
   * Cancel action
   * 
   * @return {Promise}
  */


  cancel() {
    var _this10 = this;

    return _asyncToGenerator(function* () {
      //Populate the results
      _this10.results.complete = false; //Destroy it, its cancelled

      _this10.panda.remove(); //Resolve


      return _this10.resolver(_this10.results);
    })();
  }
  /**
   * Complete
   * 
   * @return {Promise}
  */


  complete() {
    var _this11 = this;

    return _asyncToGenerator(function* () {
      //Populate the results
      _this11.results.complete = true; //Close it

      _this11.panda.remove(); //Resolve


      return _this11.resolver(_this11.results);
    })();
  }
  /**
   * Get the icons content
   * 
   * @return {Promise}
  */


  getIconContent(type) {
    return _asyncToGenerator(function* () {
      //Switch the type
      switch (type) {
        case 'close':
          return '<svg class="close-btn" viewBox="0 0 24 24" fill="none"><g id="icon/navigation/close_24px"><path id="icon/navigation/close_24px_2" d="M18.3 5.70999C18.1131 5.52273 17.8595 5.4175 17.595 5.4175C17.3305 5.4175 17.0768 5.52273 16.89 5.70999L12 10.59L7.10997 5.69999C6.92314 5.51273 6.66949 5.4075 6.40497 5.4075C6.14045 5.4075 5.8868 5.51273 5.69997 5.69999C5.30997 6.08999 5.30997 6.71999 5.69997 7.10999L10.59 12L5.69997 16.89C5.30997 17.28 5.30997 17.91 5.69997 18.3C6.08997 18.69 6.71997 18.69 7.10997 18.3L12 13.41L16.89 18.3C17.28 18.69 17.91 18.69 18.3 18.3C18.69 17.91 18.69 17.28 18.3 16.89L13.41 12L18.3 7.10999C18.68 6.72999 18.68 6.08999 18.3 5.70999Z"></path></g></svg>';
          break;
      }
    })();
  }
  /* ------------------------------------------------------------------------------------------------------------------------
      Bindings
  ------------------------------------------------------------------------------------------------------------------------ */

  /**
   * Add all of the bindings for the panda
   * 
   * @return {Promise}
  */


  bindings() {
    var _this12 = this;

    return _asyncToGenerator(function* () {
      //Let promises
      var promises = []; //Close btn?

      if (_this12.options.closeBtn) {
        promises.push(_this12.cancelBtn());
      } //Loop the slides and bind separately


      for (var [key, slide] of Object.entries(_this12.slides)) {
        //Correct the key
        key = parseInt(key) + 1;
        promises.push(_this12.nextSlideBtn(slide, key)), promises.push(_this12.previousSlideBtn(slide, key)), promises.push(_this12.customSlideBtn(slide, key));
      } //Return


      return yield Promise.all(promises);
    })();
  }

}
/**
 * Export the panda function to use
 *
 * @param {Object} slides - The slides
 * @param {Object} options - The options
 * @returns {Promise}  - An instance of panda
 */


export var Panda = /*#__PURE__*/function () {
  var _ref5 = _asyncToGenerator(function* (slides, options) {
    //Start a new Panda
    var newBuild = new Builder(slides, options); //Run the build

    return yield newBuild.create();
  });

  return function Panda(_x3, _x4) {
    return _ref5.apply(this, arguments);
  };
}();